"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Exceptions_1 = require("../Exceptions");
class Assert {
    constructor() {
        this._counts = 0;
        this._plannedCounts = 0;
        return new Proxy(this, {
            get(target, name, receiver) {
                if (typeof (target[name]) !== 'undefined') {
                    return Reflect.get(target, name, receiver);
                }
                if (typeof (chai_1.assert[name]) !== 'undefined') {
                    target._counts++;
                }
                return Reflect.get(chai_1.assert, name, receiver);
            },
        });
    }
    static use(fn) {
        return chai_1.use(fn);
    }
    plan(count) {
        this._plannedCounts = count;
    }
    evaluate() {
        if (this._plannedCounts && this._plannedCounts !== this._counts) {
            const suffix = this._plannedCounts === 1 ? '' : 's';
            const message = `Planned for ${this._plannedCounts} assertion${suffix}, but ran ${this._counts}`;
            throw new Exceptions_1.InvalidAssertionsCount(message);
        }
    }
}
exports.Assert = Assert;
