"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Exceptions_1 = require("../Exceptions");
const Debug = require("debug");
const debug = Debug('japa');
async function asPromise(fn, args) {
    return await fn(...args);
}
function Callable(resolveFn, callback, timeout) {
    return new Promise((resolve, reject) => {
        let postCallable = null;
        const args = resolveFn(done, function postRun(fn) {
            postCallable = fn;
        });
        const needsDone = args.length === callback.length;
        debug('needsDone %s', needsDone);
        let completed = false;
        let timer = null;
        function clearTimer() {
            if (timer) {
                debug('clearing timer');
                clearTimeout(timer);
            }
        }
        function finish(error) {
            if (typeof (postCallable) === 'function' && !error) {
                try {
                    postCallable(...args);
                }
                catch (postCallableError) {
                    error = postCallableError;
                }
            }
            debug(error ? 'finishing callable as error' : 'finishing callable');
            completed = true;
            clearTimer();
            if (error) {
                reject(error);
            }
            else {
                resolve();
            }
        }
        function done(error) {
            if (completed) {
                return;
            }
            if (error) {
                finish(error);
                return;
            }
            finish();
        }
        if (timeout > 0) {
            debug('creating timer');
            timer = setTimeout(() => {
                debug('timeout');
                if (!completed) {
                    finish(new Exceptions_1.TimeoutException(`Test timeout after ${timeout} milliseconds`));
                }
            }, timeout);
        }
        debug('executing callable');
        asPromise(callback, args)
            .then(() => {
            if (!needsDone) {
                finish();
            }
        })
            .catch((error) => {
            finish(error);
        });
    });
}
exports.Callable = Callable;
