"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Contracts_1 = require("../Contracts");
const Emitter_1 = require("../Emitter");
class Runner {
    constructor(_groups, _options) {
        this._groups = _groups;
        this._options = _options;
    }
    get hasErrors() {
        return !!this._groups.find((group) => group.hasErrors);
    }
    reporter(fn) {
        this._reporterFn = fn;
        return this;
    }
    async run() {
        if (typeof (this._reporterFn) !== 'function') {
            throw new Error('Make sure to define tests reporter as a function');
        }
        this._reporterFn(Emitter_1.emitter, this._options);
        Emitter_1.emitter.emit(Contracts_1.IEvents.STARTED);
        for (let group of this._groups) {
            await group.run();
            if (this._options.bail && group.hasErrors) {
                break;
            }
        }
        Emitter_1.emitter.emit(Contracts_1.IEvents.COMPLETED);
    }
}
exports.Runner = Runner;
