"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const Runner_1 = require("../Runner");
const Group_1 = require("../Group");
const Assert_1 = require("../Assert");
const list_1 = require("../Reporter/list");
const Loader_1 = require("./Loader");
const loader = new Loader_1.Loader();
function testArgsFn(done, postRun) {
    postRun(function postRunFn(assert) {
        assert.evaluate();
    });
    return [new Assert_1.Assert(), done];
}
function hookArgsFn(done) {
    return [done];
}
let groups = [];
let activeGroup = null;
let runnerOptions = {
    bail: false,
    timeout: 2000,
};
let reporterFn = list_1.default;
function addTest(title, callback, options) {
    if (!activeGroup) {
        activeGroup = new Group_1.Group('root', testArgsFn, hookArgsFn, runnerOptions);
        groups.push(activeGroup);
    }
    return activeGroup.test(title, callback, options);
}
function test(title, callback) {
    return addTest(title, callback);
}
exports.test = test;
async function run(exitProcess = true) {
    const runner = new Runner_1.Runner(groups, runnerOptions);
    runner.reporter(reporterFn);
    const loaderFiles = await loader.loadFiles();
    if (loaderFiles.length && groups.length) {
        console.log(chalk_1.default.bgRed('Calling configure inside test file is not allowed. Create a master file for same'));
        process.exit(1);
    }
    loaderFiles.forEach((file) => require(file));
    try {
        await runner.run();
        if (exitProcess) {
            runner.hasErrors ? process.exit(1) : process.exit(0);
        }
    }
    catch (error) {
        if (exitProcess) {
            process.exit(1);
        }
    }
    groups = [];
    activeGroup = null;
}
exports.run = run;
(function (test) {
    function group(title, callback) {
        activeGroup = new Group_1.Group(title, testArgsFn, hookArgsFn, runnerOptions);
        groups.push(activeGroup);
        callback(activeGroup);
        activeGroup = null;
    }
    test.group = group;
    function skip(title, callback) {
        return addTest(title, callback, { skip: true });
    }
    test.skip = skip;
    function skipInCI(title, callback) {
        return addTest(title, callback, { skipInCI: true });
    }
    test.skipInCI = skipInCI;
    function runInCI(title, callback) {
        return addTest(title, callback, { runInCI: true });
    }
    test.runInCI = runInCI;
    function failing(title, callback) {
        return addTest(title, callback, { regression: true });
    }
    test.failing = failing;
    function configure(options) {
        if (groups.length) {
            throw new Error('test.configure must be called before creating any tests');
        }
        if (options.reporterFn) {
            reporterFn = options.reporterFn;
        }
        if (options.bail !== undefined) {
            runnerOptions.bail = options.bail;
        }
        if (typeof (options.timeout) === 'number') {
            runnerOptions.timeout = options.timeout;
        }
        if (options.files !== undefined) {
            loader.files(options.files);
        }
        if (typeof (options.filter) === 'function') {
            loader.filter(options.filter);
        }
        if (options.grep) {
            runnerOptions.grep = options.grep instanceof RegExp ? options.grep : new RegExp(options.grep);
        }
    }
    test.configure = configure;
})(test = exports.test || (exports.test = {}));
